(* --- Définition du type --------------------------------*)
type 'a tas_min = (('a*float) list)ref

(* --- Fonction de construction --------------------------*)
let cree_tas_min_vide () : 'a tas_min =
	(*retourne un tas_min vide *)
	ref []


(* --- Tests de vacuité ----------------------------------*)
let est_vide (t:'a tas_min) : bool =
	(* teste si t est vide, ie n'a aucun noeud*)
	!t = []

(* --- Minimimum : élément et valeur ---------------------*)
let elem_prio_min (t:'a tas_min) : 'a*float =
	(* hyp : not(est_vide t) *)
	(* donne la valeur minimum des éléments de t *)
	let rec aux (l:('a*float)list)(res:'a)(p_res:float) : 'a*float =
		match l with
		|[] -> (res,p_res)
		|(elem,prio)::ll -> 
			if prio < p_res
			then aux (ll) (elem)(prio)
			else aux (ll) (res) (p_res)
	in match !t with 
	| [] -> failwith "pas d'élément min dans ce tas car il est vide"
	| (elem,prio)::l -> aux l elem prio
	
let elem_min (t:'a tas_min) : 'a =
	(* hyp : not(est_vide t) *)
	(* donne l'élément de priorité minimum de t *)
	let (e,p) = (elem_prio_min t) in e
	
let prio_min (t:'a tas_min) : float =
	(* hyp : not(est_vide t) *)
	(* donne la priorité minimum des éléments de t *)
	let rec aux (l: ('a*float) list) (p_res : float) : float =
		match l with
		|[] -> p_res
		|(elem,prio)::ll -> 
			if prio < p_res
			then aux ll prio
			else aux ll p_res
	in match !t with 
	| [] -> failwith "pas d'élément min dans ce tas car il est vide"
	| (elem,prio)::l -> aux l prio
	



(* --- Insertion -----------------------------------------*)
let insere_tas (t:'a tas_min) (elem_vlr:'a*float) : unit =
	(* insère un elem de valeur vlr dans le tas t *)
	t:= elem_vlr::(!t)
	


(* --- Suppression ---------------------------------------*)
let rec transvase (l:('b list)ref)(l_bis:'b list):unit =
	(* tranvase les éléments de l_bis dans l, à l'envers*)
	match l_bis with
	| [] -> ()
	| a::q -> l:= a::(!l); transvase l q
	
	
let rec supprime_ind (l:('b list)ref)(l_bis:'b list)(k:int):unit =
		(*hyp : k est un indice de ll*)
		(* calcule une liste contenant les éléments de l_bis *)
		(* et ceux de ll sauf celui d'indice l *)
		match k,!l with
		| _,[] -> failwith ("indice à supprimer non valide")
		| 0,(e::ll) -> l:=ll; transvase l l_bis
		| _,(e::ll) -> l:=ll; supprime_ind l (e::l_bis) (k-1) 
	
let supprime_min (t:'a tas_min) : unit =
	(* hyp : not (est_vide t) *)
	(* Supprime du tas t l'élément de valeur minimale *)
	let rec ind_min (l:('a*float)list)(p:float)(k:int)(k_min:int):int =
		(*calcule k+l'indice de la prio minimum dans ll si elle est <p *)
		(* et k_min sinon *)
		match l with
		| [] -> k_min
		| (_,prio)::ll ->
			if prio < p 
			then ind_min ll prio (k+1) k
			else ind_min ll p (k+1) k_min
	in match !t with
	| [] -> failwith "on ne peut supprimer dans un tas vide "
	| (elem,prio)::ll -> supprime_ind t [] (ind_min ll prio 1 0)
	

	
(* --- Pour petit test -----------------------------------*)
let t0 = cree_tas_min_vide ()
let construit_t : unit =
	insere_tas t0 ('a',2.9);
	insere_tas t0 ('b',3.9);
	insere_tas t0 ('c',5.);
	assert (prio_min t0 = 2.9); (*attention test sur float dangereux*)
	assert (elem_min t0 = 'a');
	supprime_min t0; 
	insere_tas t0 ('d',3.5);
	assert (elem_min t0 = 'd')
	
	
	
	
	
